@@ -9,7 +9,7 @@
 # 
 package Config::Model::Tk::HashEditor ;
 BEGIN {
-  $Config::Model::Tk::HashEditor::VERSION = '1.312';
+  $Config::Model::Tk::HashEditor::VERSION = '1.313';
 }
 
 use strict;
@@ -212,6 +212,7 @@ sub Populate {
     # set all buttons to their default state
     $cw->update_state(tklist => '', entry => '') ;
 
+    $cw->add_warning($hash)->pack(@fx) ;
     $cw->add_info_button()->pack( @fx,qw/-anchor n/) ;
     $cw->add_summary($hash)->pack(@fx) ;
     $cw->add_description($hash)->pack(@fx) ;
@@ -516,6 +517,7 @@ sub store {
 
 sub reload_tree {
     my $cw = shift ;
+    $cw->update_warning($cw->{hash}) ;
     $cw->{store_cb}->() ;
 }
 
